#ifndef TEMPUS_VALIDITY_PERIOD_HH
#define TEMPUS_VALIDITY_PERIOD_HH

#include "property.hh"
#include "base.hh"
#include "datetime.hh"

namespace Tempus
{

///
/// Describe a period of validity for some property associated to road sections.
/// e.g. speed profiles or road restrictions
class ValidityPeriod : public Base
{
public:
    DECLARE_RW_PROPERTY_DEFAULT( on_monday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_tuesday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_wednesday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_thursday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_friday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_saturday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( on_sunday, bool, true );
    DECLARE_RW_PROPERTY_DEFAULT( start_date, Date, Date(boost::date_time::neg_infin) );
    DECLARE_RW_PROPERTY_DEFAULT( end_date, Date, Date(boost::date_time::pos_infin) );

    DECLARE_RW_PROPERTY_DEFAULT( name, std::string, "" );

    ///
    /// Test if a given date is included in the period
    bool is_valid_for( const Date& date ) const;
};

using ValidityPeriods = std::map<db_id_t, ValidityPeriod>;

}
#endif
